package PlayBot::sessions::irc;

use strict;
use warnings;
use utf8;

use DBI;
use POE;
use POE::Component::IRC::State;
use POSIX 'strftime';
use Tie::File;
use JSON;
use Module::Reload;

use PlayBot::utils::Logging;
use PlayBot::sessions::irc::later;
use PlayBot::sites;
use PlayBot::sites::soundcloud;
use PlayBot::commands::parser;

my $log = PlayBot::utils::Logging->new('STDOUT', 1);

local $/;
open CONF, '<', 'playbot.conf';
my $json = <CONF>;
my $conf = decode_json($json);

## CONNEXION
my ($irc) = POE::Component::IRC::State->spawn();
my $dbh = DBI->connect('DBI:mysql:'.$conf->{'bdd'}.';host='.$conf->{'host'}, $conf->{'user'}, $conf->{'passwd'}, {
	    PrintError => 0,
	    AutoCommit => 1,
		mysql_auto_reconnect => 1
	})
	or die("Couldn't connect to database: ".DBI->errstr);

# config
my $serveur = '193.54.225.86';
my $nick = $conf->{'nick'};
my $port = 6667;
my $ircname = 'nightiies';
my $username = 'nightiies';
my @channels = @{ $conf->{'channels'} };
my $admin = 'moise';
my $baseurl = 'http://nightiies.iiens.net/links/';
my @nicksToVerify;
my @codesToVerify;
my %lastID;

my $debug = 0;

# mode debug
if ($#ARGV + 1) {
	@channels = ('#hormone');
	$nick = 'kikoo';
	$debug = 1;
}

# Evenements que le bot va gérer
POE::Session->create(
	inline_states => {
        _start     => \&bot_start,
        irc_001    => \&on_connect,
        irc_public => \&on_speak,
        irc_msg    => \&on_query,
        irc_invite => \&on_invite,
        irc_notice => \&on_notice,
		_flux	   => \&flux,
        _later     => sub {
            PlayBot::sessions::irc::later::consume(@_[ARG0, ARG1, ARG2])
        }
	},
);


my %commandes_admin = ("cycle" => \&cycle);


### FONCTIONS
sub flux
{
	my $kernel = $_[ KERNEL ];
	my $date = strftime ("%Y-%m-%d", localtime(time - 3600*24));

    foreach (@channels) {
	    my $sth = $dbh->prepare('
            SELECT COUNT(*)
            FROM playbot p
            JOIN playbot_chan pc ON p.id = pc.content
            WHERE date = ? and chan = ?');
	    $log->error("Couldn't prepare querie; aborting") unless (defined $sth);
	    $sth->execute($date, $_)
		    or $log->error("Couldn't finish transaction: " . $dbh->errstr);
	    my ($nbr) = $sth->fetchrow_array;

	    if ($nbr) {
		    $irc->yield(privmsg => $_ => $nbr.' liens aujourd\'hui : '.$baseurl . substr ($_, 1) . '/' . $date);
	    }
    }

	$kernel->delay_set('_flux', 3600*24);
}


sub cycle
{
	my ($arg) = @_;

	$log->info("refresh modules");

    Module::Reload->check;

    setConf();
}


sub setConf
{
    PlayBot::commands::parser::setConf($irc, $dbh, $log, \%lastID);

    $PlayBot::sites::irc = $irc;
    $PlayBot::sites::log = $log;
    $PlayBot::sites::playbot_api_key = $conf->{'playbot_api_key'};
	$PlayBot::sites::soundcloud::clientSecret = $conf->{'soundcloud_secret'};
}


## GESTION EVENTS

# Au démarrage
sub bot_start {
    my $kernel = $_[KERNEL];
    $kernel->alias_set('bot');

    $log->info("register all");
	$irc->yield(register => "all");
    $log->info("connecting");
	$irc->yield(
		connect => {
			Nick     => $nick,
			Username => $username,
			Ircname  => $ircname,
			Server   => $serveur,
			Port     => $port,
		}
	);
}


# A la connection
sub on_connect
{
	my $kernel = $_[ KERNEL ];

	$log->info('connected');
    setConf();
    $irc->yield(privmsg => "NickServ" => "identify ".$conf->{'nickserv_pwd'}) unless ($debug);

	foreach (@channels) {
		$irc->delay([join => $_], 20);
		$log->info("join $_");
	}

	my $hour = strftime ('%H', localtime);
	my $min = strftime ('%M', localtime);

	#$kernel->delay_set('_flux', (23-$hour)*3600 + (60-$min)*60);
}


# Discussion privée
sub on_query
{
	my ($kernel, $user, $msg) = @_[KERNEL, ARG0, ARG2];
	my ($nick) = split (/!/,$user);

    my $fake_chan = $nick;
    my @args = ($kernel, $user, $fake_chan, $msg);

    return if (PlayBot::commands::parser::exec(@args));

	if ($msg =~ m/^!/ && $nick eq $admin) {
		my $commande = ( $msg =~ m/^!([^ ]*)/ )[0];
		my @params = grep {!/^\s*$/} split(/\s+/, substr($msg, length("!$commande")));

		foreach (keys(%commandes_admin)) {
			if ($commande eq $_) {
				$commandes_admin{$_}->(@params);
				last;
			}
		}
	}
	elsif ($msg =~ /^PB/) {
		# on vérifie si le nick est register
		push (@nicksToVerify, $nick);
		push (@codesToVerify, $msg);
		$irc->yield(privmsg => $nick => 'Vérification en cours…');
		$irc->yield(privmsg => nickserv =>  'info '.$nick);
	}
}


sub on_notice
{
	my ($user, $msg) = @_[ARG0, ARG2];
	my ($nick) = split(/!/,$user);

	return unless ($nick =~ /^NickServ$/i);

	my $nickToVerify = shift @nicksToVerify;
	my $code = shift @codesToVerify;

	return unless (defined($nickToVerify));

	if ($msg !~ /$nickToVerify/) {
		push (@nicksToVerify, $nickToVerify);
		push (@codesToVerify, $code);
	}
	elsif ($msg =~ /isn't registered/) {
		$irc->yield(privmsg => $nickToVerify => "Il faut que ton pseudo soit enregistré auprès de NickServ");
	}
	else {
		my $sth = $dbh->prepare('SELECT user FROM playbot_codes WHERE code = ?');
		$log->error("Counldn't prepare querie; aborting") unless (defined $sth);
		$sth->execute($code);

		if ($sth->rows) {
			my $sth = $dbh->prepare('UPDATE playbot_codes SET nick = ? WHERE code = ?');
			$log->error("Couldn't prepare querie; aborting") unless (defined $sth);

			$sth->execute($nickToVerify, $code)
				or $log->error("Couldn't finish transaction: " . $dbh->errstr);

			$irc->yield(privmsg => $nickToVerify => 'Association effectuée');
			$irc->yield(privmsg => $nickToVerify => 'pour enregistrer un lien dans tes favoris : !fav <id>');
		}
		else {
			$irc->yield(privmsg => $nickToVerify => "Ce code n'existe pas");
		}
	}
}

# Quand on m'invite, je join
sub on_invite
{
	my ($kernel, $user, $chan) = @_[KERNEL, ARG0, ARG1];
	my ($nick,$mask) = split(/!/,$user);

	$log->info($nick . " m'invite sur ". $chan);
	$irc->yield(join => $chan);

    push @channels, $chan;
}

# Quand un user parle
sub on_speak
{
	my ($kernel, $user, $chan, $msg) = @_[KERNEL, ARG0, ARG1, ARG2];
    my @args = ($kernel, $user, lc $chan->[0], $msg);

	my ($nick,$mask) = split(/!/,$user);
	my %content;

    # first we check for url
    my @processed_str = PlayBot::sites::parse(@args);

    # we remove any part processed (url and tags)
    foreach (@processed_str) {
        $msg =~ s/\Q$_\E// if $_;
    }

    # and we check for command
    @args = ($kernel, $user, lc $chan->[0], $msg);
    PlayBot::commands::parser::exec(@args);
}

1;
